/*
% A basic textured mapping shader with ambient.
author: Alex Lemke
date: 4/20/12
*/

#define FLIPTEXTURE_Y 1

float4x4 WorldViewProj : WORLDVIEWPROJECTION < string UIWidget = "none"; >;

float4 AmbientColor : COLOR < string UIWidget = "Color"; > = {0.1, 0.1, 0.1, 0.1};

texture ColorTexture;
sampler ColorTextureSampler : register(s0) = sampler_state
{
	Texture = (ColorTexture);
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = Mirror;
	AddressV = Mirror;
};

struct VS_IN
{
	float4 Position : POSITION0;
	float2 TexCoord : TEXCOORD0;
};

struct VS_OUT
{
	float4 Position : POSITION0;
	float2 TexCoord : TEXCOORD0;
};

VS_OUT mainVS(VS_IN input)
{
	VS_OUT output;
	
	output.Position = mul(float4(input.Position.xyz, 1.0), WorldViewProj);
	
	#if FLIPTEXTURE_Y
		output.TexCoord = float2(input.TexCoord.x, 1 - input.TexCoord.y);
	#else
		output.TexCoord = input.TexCoord;
	#endif
	
	return output;
}

float4 mainPS(VS_OUT input) : COLOR 
{
	float4 Color = tex2D(ColorTextureSampler, input.TexCoord);
	Color.rgb *= (AmbientColor.rgb * AmbientColor.a);
	return Color;
}

technique technique0 {
	pass p0 {
		CullMode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_0 mainPS();
	}
}
