/*
% A basic textured mapping shader with ambient light and directional light.
author: Alex Lemke
date: 4/20/12
*/

#include "include\Common.fxh"

float4x4 WorldViewProj : WORLDVIEWPROJECTION < string UIWidget = "none"; >;
float4x4 WorldInverseTranspose : WORLDINVERSETRANSPOSE < string UIWidget = "none"; >;

float4 AmbientColor : COLOR < string UIWidget = "Color"; > = {0.1, 0.1, 0.1, 0.1};

float3 DirectionalLightDirection : DIRECTION < string Object = "DirectionalLight"; string UIName = "DirectionalLight 0 Direction"; string Space = "World"; >;
float4 DirectionalLightColor : DIFFUSE < string Object = "DirectionalLight"; >;

texture ColorTexture;
sampler ColorTextureSampler : register(s0) = sampler_state
{
	Texture = (ColorTexture);
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = Mirror;
	AddressV = Mirror;
};

struct VS_IN
{
	float4 Position : POSITION0;
	float2 TexCoord : TEXCOORD0;
	float3 Normal 	: NORMAL;
};

struct VS_OUT
{
	float4 Position 		: POSITION0;
	float2 TexCoord 		: TEXCOORD0;
	float3 LightDirection 	: TEXCOORD1;
	float3 Normal 			: TEXCOORD2;
};

VS_OUT mainVS(VS_IN input)
{
	VS_OUT output;
	
	output.Position = mul(float4(input.Position.xyz, 1.0), WorldViewProj);
	output.TexCoord = get_corrected_texture_coord(input.TexCoord);
	output.Normal = mul(float4(input.Normal.xyz, 1.0), WorldInverseTranspose);
	output.LightDirection = normalize(-DirectionalLightDirection);
	
	return output;
}

float4 mainPS(VS_OUT input) : COLOR 
{
	//Normalizing Inputs
	float3 normal = normalize(input.Normal);
	
	//Light
	input.LightDirection = normalize(input.LightDirection);
	float NdotL = saturate(dot(normal, input.LightDirection));	
	
	float4 textureColor = tex2D(ColorTextureSampler, input.TexCoord);
	float4 ambient = get_color_contribution(AmbientColor, textureColor);
	float4 diffuse = float4((DirectionalLightColor.rgb * DirectionalLightColor.a), 1.0f) * NdotL * textureColor;
	
	return diffuse + ambient;
}

technique technique0 {
	pass p0 {
		CullMode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_0 mainPS();
	}
}
