/*
% A skybox shader.
author: Alex Lemke
date: 4/28/12
*/

float4x4 WorldViewProj : WORLDVIEWPROJECTION < string UIWidget = "none"; >;
float4x4 World : WORLD < string UIWidget = "none"; >;
float4x4 View : VIEW < string UIWidget = "none"; >;

//Ambient Color
float4 AmbientLightColor : Color;

texture SkyboxTexture; 
samplerCUBE SkyboxSampler : register(s0) = sampler_state 
{ 
   texture = (SkyboxTexture); 
   magfilter = LINEAR; 
   minfilter = LINEAR; 
   mipfilter = LINEAR; 
   AddressU = Mirror; 
   AddressV = Mirror; 
};

struct VS_IN
{
	float4 ObjectPosition : POSITION0;
};

struct VS_OUT
{
	float4 ScreenPosition : POSITION0;
	float3 TexCoord : TEXCOORD0;
};

VS_OUT mainVS(VS_IN In)
{
	VS_OUT Out = (VS_OUT)0;

    Out.ScreenPosition = mul(In.ObjectPosition, WorldViewProj);
	
	float4 worldPosition = mul(In.ObjectPosition, World);
	float3 eyePosition = View[3].xyz;
	Out.TexCoord = normalize(worldPosition - eyePosition);
	
	return Out;
}

float4 mainPS(VS_OUT In) : COLOR {
	float4 color = texCUBE(SkyboxSampler, normalize(In.TexCoord)) * AmbientLightColor;
	return color;
}

technique technique0 {
	pass p0 {
		CullMode = None;
		VertexShader = compile vs_3_0 mainVS();
		PixelShader = compile ps_3_0 mainPS();
	}
}
