/*
% A basic non-dynamic environmental mapping shader.
author: Alex Lemke
date: 4/20/12
*/

float4x4 world : World;
float4x4 worldViewProj : WorldViewProjection;
float4x4 worldInverseTranspose : WorldInverseTranspose;

//Ambient Color
float3 ambientLightColor : Color;

//Camera Position
float3 cameraPosition : CameraPosition;

texture CubeMapTexture;
samplerCUBE CubeMapTextureSampler = sampler_state
{
	texture = <CubeMapTexture>; 
	magfilter = LINEAR; 
   	minfilter = LINEAR; 
   	mipfilter = LINEAR; 
   	AddressU = Mirror; 
   	AddressV = Mirror; 
};

/**float3 reflect(float3 I, float3 N)
{
	return I - 2 * N * dot(I, N);  // R = I - 2 * N * (I.N)
}

float3 refract(float3 I, float3 N, float ri)
{
	float cosI = dot(-I, N);
	float cosT = 1.0f - ri * ri * (1.0f - cosI * cosI);
	
	float3 T = ri * I + ((ri * cosI - sqrt(abs(cosT))) * N);
	
	return T * (float3)(cosT > 0);
	
}*/


struct VS_IN
{
	float4 Position : POSITION0;
	float2 TexCoord : TEXCOORD0;
	float3 Normal	: NORMAL;
};

struct VS_OUT
{
	float4 Position : POSITION0;
	float3 Reflect 	: TEXCOORD0;
	float3 Refract	: TEXCOORD1;

};

VS_OUT mainVS(VS_IN input)
{
	VS_OUT output = (VS_OUT)0;
	
	output.Position = mul(float4(input.Position.xyz, 1.0), worldViewProj);
		
	input.Normal = normalize(input.Normal);
	input.Normal = mul(float4(input.Normal.xyz, 1.0), world);
	
	
	// Determine the direction from the view to the vertex
	float3 viewDirection = -normalize(cameraPosition - input.Position);
	
	//Get a vector towards the camera eye
	float3 PosWorld = normalize(mul(input.Position.xyz, world));
	float3 Incident = normalize(PosWorld - viewDirection);
	
	float3 ShortNormal = mul(input.Normal, 0.4);
	
	// Reflection Vector for cube map: R = I - 2*N * (I.N)
	output.Reflect = reflect(viewDirection, input.Normal);
	
	reflect(-normalize(viewDirection), normalize(input.Normal));
	
	// approximated refracted vector
	output.Refract = reflect(Incident, ShortNormal);
	
	//  Snell's law in refract()
	output.Refract = refract(Incident, input.Normal, 0.99f);
	
	return output;
}

float4 mainPS(VS_OUT In) : COLOR 
{
	float4 tex1 = texCUBE(CubeMapTextureSampler, In.Reflect);
	float4 tex2 = texCUBE(CubeMapTextureSampler, In.Refract);	
	return tex2 * 0.9 + tex1;
}

technique technique0 {
	pass p0 {
		CullMode = None;
		VertexShader = compile vs_3_0 mainVS();
		PixelShader = compile ps_3_0 mainPS();
	}
}

