/*
% A basic textured shader.
author: Alex Lemke
date: 4/20/12
*/

#define FLIPTEXTURE_Y 0

float2 get_corrected_texture_coord(float2 texCoord)
{
	#if FLIPTEXTURE_Y
		return float2(texCoord.x, 1 - texCoord.y);
	#else
		return texCoord;
	#endif
}

float4 get_color_contribution(float4 contributingColor, float4 surfaceColor)
{
	return float4(contributingColor.rgb * contributingColor.a, 1.0f) * surfaceColor;
}

float get_fog_amount(float3 cameraPosition, float4 worldPosition, float fogStart, float fogEnd)
{
	return saturate((length(cameraPosition - worldPosition) - fogStart) / (fogEnd - fogStart));
}

float3 apply_fog(float3 fogColor, float4 surfaceColor, float fogAmount)
{
	return lerp(surfaceColor, fogColor, fogAmount);
}

float4 get_light_data(float3 lightPosition, float4 worldPosition, float lightRadius)
{
	float4 returnValue;
	
	float3 lightDirection = lightPosition - worldPosition;

	returnValue.xyz	= -normalize(lightDirection);
	returnValue.w = saturate(1.0f - length(lightDirection) / lightRadius); // Attenuation
	
	return returnValue;
}

float4 get_light_data(float3 lightPosition, float4 worldPosition, float lightRadius, float3x3 worldToTangentSpace)
{
	float4 lightData = get_light_data(lightPosition, worldPosition, lightRadius);
	lightData.xyz = mul(lightData.xyz, worldToTangentSpace);
	return lightData;
}

struct LightingCoefficientStruct
{
	float4 LightDirection;
	float3 Normal;
	float3 View;
	float4 SurfaceColor;
	float4 LightColor;
	float4 SpecularColor;
	float SpecularPower;
};


float4 calculate_lighting(LightingCoefficientStruct input)
{	
	float3 lightDirection = -normalize(input.LightDirection.xyz);
	
	// Some Vector Stuff
	float NdotL = saturate(dot(lightDirection,  input.Normal));
	float3 halfVector = normalize( input.View + lightDirection);
	float NdotH = dot( input.Normal, halfVector);
	
	//Calc lighting
	float4 lightCoefficients = lit(NdotL, NdotH,  input.SpecularPower);
	
	float4 diffuse = get_color_contribution(input.LightColor, lightCoefficients.y *  input.SurfaceColor);
	float4 specular = get_color_contribution(input.SpecularColor, min(lightCoefficients.z,  input.SurfaceColor.w));
	
	float4 output = (get_color_contribution(input.LightColor, (diffuse + specular)) * input.LightDirection.w);
	
	
	return output;
}

float3x3 world_to_tangent_space(float3 tangent, float3 normal, float4x4 world)
{
	float3x3 worldToTangentSpace;
	worldToTangentSpace[0] = mul(tangent, world);
    worldToTangentSpace[1] = mul(cross(tangent, normal), world);
    worldToTangentSpace[2] = mul(normal, world);
	return worldToTangentSpace;
}

///////////////////////////////////////////////////////////////
// Clears - Used to clear buffers and render targets
///////////////////////////////////////////////////////////////

float4 gClearColor < string UIWidget = "Color"; string UIName = "Background"; > = {0,0,0,0};
float gClearDepth < string UIWidget = "none";> = 1.0;
float4 gShadowMapClearColor < string UIWidget = "none"; > = {0.0,0.0,0.0,0.0};

